
warning off;

//------------------declaration-----------------//

var
    
    C                        //1
    L                         //2
    Lam 
    mu 
    w 
    RF 
    RB 
    Q 
    QB 
    f 
    b 
    re 
    n 
    lam 
    omega 
    Omega 
    theta 
    phi 
    d 
    Pi 
    pr 
    x1 
    x2
    vp 
    pm 
    Ym 
    Y 
    K 
    A 
    fm 
    Ih 
    v1 
    v2 
    pk 
    I 
    RTR 
    Rre 
    Rd 
    fcb 
    bcb 
    G 
    T 
    xi 
    welf 
    epsi 
    Psi
    spreadf 
    spreadb 
    gY 
    gC 
    gI 
    gf 
    gb 
    gbcb;

varexo
    eA                     // productivity growth shock
    eG                     // government purchase shock
    et                      // credit shock
    ef                      // QE private bond purchase
    eb                     // QE government bond purchase
    er                      // policy rate shock
    exi                     // bond price shock
    em                     // markup shock
    ep                      // borrowing constraint shock
 ;
  
parameters
    // --------calibrated---------------------------
    sp
    Pis 
    RTRs 
    kappa 
    alpha 
    delta 
    Psis 
    fcbs 
    bcbs 
    bG 
    Gs 
    As 
    xis 
    epsis 
    kI  
    Ys 
    fms 
    Kai 
    gy
    fracb
    fracf
    bgy
    lev
    //------------- estimated parameters--------------------
    beta                  //discount factor
    bh                     //consumption habit
    thetas                //credit steadystate
    Delta                 // ratio of government bond credit spread of private
    X                       // starup capital
    sigma                // survival rate
    phip                   
    gammap             // inflation stickness
    eta                      // inverse of frisch elasticity
    v                         // MP exponential
    rhor                    // poicy persistence
    phipi                   // Taylor weights on inflation
    phiy                    // Taylor weights on output
    rhof                    // QE private persistence
    rhob                    // QE government persistence
    rhoG                   // government purchase persistence
    rhoA                   // productivity persistence
    rhot                    // credit shock persistence
    rhoxi                  // capital price shock persistence
    sr                       // policy shock std
    sf                       // QE private shock std
    sb                       // QE bond shock std
    sA                       // productivity shock std
    sG                       // government purchase std
    st                        // credit shock std
    sxi                       // markup shock std
;

//------------------------ calibration --------------------//

beta = 0.95;             % discount factor
Pis = 1;                  % steady state gross inflation
bh=0;                    % habit formation
kappa = 1-40^(-1);        % decay parameter for bond
Ls=1;                     % steady state labor
%phis=3;                   % steady state endogenous leverage
lev=5;
sigma = 0.95;             % survival probability
sp=0.03;                % steady state spread
Delta = 1/3;              % steady state difference in recoverability government vs private
epsis = 11;                % elasticity goods
phip = 0.75;              % price stickiness
gammap = 0;               % price indexation
alpha = 0.4;             % production function
delta = 0.025;            % depreciation rate
Psis = .8;                 % fraction of investment from debt
kI=2;                     % adjustment cost parameter
rhor=0.9;                 % Taylor rule smoothing
phipi=1.5;                % Taylor rule inflation
phiy=0.125;                % Taylor rule output
bgy=0.3*4;                % steady state debt-gdp ratio
gy=0.2;                   % government spending share
As=1;                     % steady state technology
xis=1;                    % steady state capital price shock
eta=2;                    % Inverse Frisch elastisity 
v=3;





% shock parameters
rhoA = 0.9;
rhoG = 0.95;
rhof = 0.7;
rhob = 0.9;
rhoxi=0.8;        % AR(1) capital shock
rhot=0.8;

sA = 0.1;
sG = 0.01; 
sr =  0.0025; 
sf = 0.0025; 
sb = 0.0025;
sxi=0.0025;
st=0.04;
sv=0.04;
rhof= 0.9;
rhob= 0.9;
fracb = 0.06*4;
fracf = 0;

 //-------------------- model--------------------------------//

 model;
 %%%%%%%%%%%%%%%%% Household %%%%%%%%%%%%%%%%%
% (1) marginal utility
exp(mu)=(exp(C)-bh*exp(C(-1)))^(-1)-bh*beta/(exp(C(+1))-bh*exp(C));

% (2) Lam
exp(Lam)=beta*exp(mu)/exp(mu(-1));

% (3) Euler equation
1=Rd*exp(Lam(+1))/exp(Pi(+1));

% (4) labor supply
Kai*exp(eta*L)=exp(mu)*exp(w);

%%%%%%%%%%%%%%%%%%%% Financial intermediaries %%%%%%%%%%%%%%%%%%%
% (5) RF
exp(RF)=(1+kappa*exp(xi)*exp(Q))/(exp(xi(-1))*exp(Q(-1)));

% (6) RB
exp(RB)=(1+kappa*exp(xi)*exp(QB))/(exp(xi(-1))*exp(QB(-1)));

% (7) Balance sheet
exp(xi)*exp(Q)*exp(f)+Delta*exp(xi)*exp(QB)*exp(b)+exp(re)=exp(d)+exp(n);

% (8) Evolution of net worth
exp(n)*exp(Pi)=sigma*((exp(RF)-Rd(-1))*exp(xi(-1))*exp(Q(-1))*exp(f(-1))   +     (exp(RB)-Rd(-1))*exp(xi(-1))*exp(QB(-1))*exp(b(-1))  
+    (Rre(-1)-Rd(-1))*exp(re(-1))  +Rd(-1)*exp(n(-1))) +X;

%exp(n(+1))*exp(Pi(+1))=sigma*((exp(RF(+1))-Rd)*exp(xi)*exp(Q)*exp(f)   +     (exp(RB(+1))-Rd)*exp(xi)*exp(QB)*exp(b)  
%+    (exp(Rre)-exp(Rd))*exp(re)  +exp(Rd)*exp(n)) +X;


% (9) Omega
exp(Omega)=1-sigma+sigma*exp(theta)*exp(phi);

% (10) FOC for f
exp(Lam(+1))*exp(Omega(+1))/exp(Pi(+1))*(exp(RF(+1))-Rd)=exp(lam)*exp(theta)/(1+exp(lam));

% (11) FOC for b
exp(Lam(+1))*exp(Omega(+1))/exp(Pi(+1))*(exp(RB(+1))-Rd)=Delta*exp(lam)*exp(theta)/(1+exp(lam));

% (12) FOC for re
exp(Lam(+1))*exp(Omega(+1))/exp(Pi(+1))*(Rre-Rd)=-omega/(1+exp(lam));

% (13) Endogenous leverage
exp(theta)*exp(phi)=(1+exp(lam))*exp(Lam(+1))*exp(Omega(+1))/exp(Pi(+1))*Rd-omega*exp(re)/exp(n);

% (14) Financial constraint
exp(phi)=(exp(xi)*exp(Q)*exp(f)+Delta*exp(QB)*exp(b))/exp(n);

%%%%%%%%%%%%%%%%%%% Calvo pricing %%%%%%%%%%%%%%%%%%%%%%%%%
% (15) reset price
exp(pr)=epsi/(epsi-1)*exp(x1)/exp(x2);

% (16) Auxiliary price-setting x1
exp(x1)=exp(pm)*exp(Y)+phip*exp(Lam(+1))*(exp(Pi(+1))/exp(Pi*gammap))^epsi*exp(x1(+1));

% (17) Auxiliary price-setting x2
exp(x2)=exp(Y)+phip*exp(Lam(+1))*(exp(Pi(+1))/exp(Pi*gammap))^(epsi-1)*exp(x2);

% (18) Y and Ym
exp(vp)*exp(Y)=exp(Ym);

% (19) price dispersion
exp(vp)=(1-phip)*exp((-epsi)*pr)+phip*(exp(Pi)/exp(gammap*(Pi(-1))))^epsi*exp(vp(-1));

% (20) price evolution
1=(1-phip)*exp((1-epsi)*pr)+phip*exp(gammap*(1-epsi)*Pi(-1))*exp(Pi*(epsi-1));

%%%%%%%%%%%%%%%% Firm %%%%%%%%%%%%%%%%%%%%%%
% (21) Production function
exp(Ym)=exp(A)*exp(K(-1)*alpha)*exp(L*(1-alpha));

% (22) CaPital accumulation
exp(K)=exp(Ih)+(1-delta)*exp(K(-1));



% (24) FOC for capital
exp(Lam(+1))*(alpha*exp(pm(+1))*exp(Ym(+1))/exp(K)+exp(v1(+1))*(1-delta))=exp(v1);

% (25) FOC for labor
(1-alpha)*exp(pm)*exp(Ym)/exp(L)=exp(w);

% (26) FOC for investment
[name='borrowing',bind='ZLB']
v2=0;

[name='borrowing',relax='ZLB']
exp(xi)*exp(Q)*(1+v2)=exp(Lam(+1))/exp(Pi(+1))*(1+kappa*exp(xi(+1))*exp(Q(+1))*(1+v2(+1)));

% (23) financial constraint
Psi*exp(pk)*exp(Ih)=exp(xi)*exp(Q)*(exp(fm)-kappa*exp(fm(-1))/exp(Pi));


% (27) FOC for fm
exp(v1)=Psi*v2*exp(pk)+exp(pk);

%%%%%%%%%%%%%%%%%%%% Capital production %%%%%%%%%%%%%
% (28) capital production
exp(Ih)=(1-kI/2*(exp(I)/exp(I(-1))-1)^2)*exp(I);

% (29) FOC for capital
1=exp(pk)*(1-kI/2*(exp(I)/exp(I(-1))-1)^2-kI*(exp(I)/exp(I(-1))-1)*(exp(I)/exp(I(-1))))+exp(Lam(+1))*exp(pk(+1))*kI*(exp(I(+1))/exp(I)-1)*(exp(I(+1))/exp(I))^2;

%%%%%%%%%%%%%%%%%%%% monetary policy %%%%%%%%%%%%%%%%%
% (30) Taylor rule
RTR=(1-rhor)*RTRs+rhor*RTR(-1)+(1-rhor)*(phipi*(Pi-log(Pis))+phiy*(Y-log(Ys)))+sr*er;

% (31) Rre


[name='deposite rate',bind='rate']
Rre=1;

[name='deposite rate',relax='rate']
Rre=RTR;

% (32) Rd
Rd=Rre;

%%%%%%%%%%%%%%%%%%%%% QE %%%%%%%%%%%%%%%%%%%%%%%%%%
% (33) QE
exp(xi)*exp(Q)*fcb+exp(xi)*exp(QB)*exp(bcb)=exp(re);

% (34) fcb
fcb=(1-rhof)*fcbs+rhof*fcb(-1)+sf*ef;
%fcb=(1-rhof)*fcbs+rhof*fcb(-1)+(1-rhof)*(phipi*(Pi-log(Pis))+phiy*(Y-log(Ys)))+sf*ef;

% (35) bcb
exp(bcb)=(1-rhob)*bcbs+rhob*exp(bcb(-1))+sb*eb;

%%%%%%%%%%%%%%%%%%%%% Fiscal policy %%%%%%%%%%%%%%%%%%%%%
% (36) Budget constraint
exp(G)+bG/exp(Pi)=T+exp(xi)*exp(QB)*bG*(1-kappa/exp(Pi));

% (37) Government purchase
G=(1-rhoG)*log(Gs)+rhoG*G(-1)+sG*eG;

%%%%%%%%%%%%%%%%%%%%%%% Exogenous shock%%%%%%%%%%%%%%%%%%%%
% (38) A
A = (1-rhoA)*log(As)+rhoA*A(-1)+sA*eA;

% (39) theta
theta = (1-rhor)*log(thetas)+rhor*theta(-1)+et*st;

%%%%%%%%%%%%%%%%%%%%%%% Market clearing condition %%%%%%%%%%%%%%%%%
% (40) Bond market
exp(fm)=exp(f)+fcb;

% (41) Government Bond market
bG=exp(b)+exp(bcb);

% (42) Resource constraint
exp(Y)=exp(C)+exp(I)+exp(G);

% (43) xi
xi=(1-rhoxi)*log(xis)+rhoxi*xi(-1)-sxi*exi;

% (44) Welfare
welf = log(exp(C) - bh*exp(C(-1))) - Kai*exp(L)^(1+eta)/(1+eta) + beta*welf(+1);

% (45) markup shock
epsi=(1-rhor)*epsis+rhor*epsi(-1)+em;

%Psi=(1-rhor)*Psis+rhor*Psi(-1)+ep;
%Psi=Psis*((exp(fm))/fms)^(-v)+ep;
Psi=(1-rhor)*Psis*((exp(fm))/fms)^(-v)+rhor*Psi(-1)+ep;

% (46) spread
spreadf = RF - log(Rd);

% (47)
spreadb = RB - log(Rd);

%%%%%%%%%%%% 6 more growth rate variables :gY gC gI gfcb gf gb
%% steady state: 
%% gY gC gI gf gb=0 
%% gfcb=1

% (48) GDP growth rate
gY=Y-Y(-1);

% (49) consumption growth rate
gC=C-C(-1);

% (50) Investment 
gI=I-I(-1);

% (51) fcb
gbcb=bcb-bcb(-1);

% (52) f
gf=f-f(-1);

% (53) b
gb=b-b(-1);
 

 end;
//-----------------------check-------------------------------//
 steady;
 check;
 //---------------------simulation--------------------------------//
shocks(surprise);
%%%%% MP shocks
var et;
periods 1:5;
values (5);
var er;
periods 1:20;
values (-10);
end;
//------------------------occbin constraints---------------------//
occbin_constraints;
    name 'ZLB'; bind v2<0; relax exp(xi)*exp(Q)*(exp(fm)-kappa*exp(fm)/exp(Pi))>Psi*exp(pk)*exp(Ih);
    name 'rate'; bind RTR<=1; relax RTR>1;
end;

// ---------------- Estimation -----------------------------------------//        

// -----------------Occbin ----------------------------------------------//   
occbin_setup(simul_periods=50);
occbin_solver;
occbin_graph Y C Rre Psi fcb v2 levf fm;
    